"""
Log out from conda repo
"""
import logging

from ..utils.config import remove_token
from .base import SubCommandBase

logger = logging.getLogger("repo_cli")


class SubCommand(SubCommandBase):
    name = "logout"

    def main(self):
        if self.access_token or self.args.force:
            # call remove token that will remove the token from the current selected site...
            remove_token(self.args)
            if self.args.force:
                self.log.info("Access tokens were removed")
            else:
                self.log.info("Logout successful")
        else:
            self.log.info("You are not logged in")

    def add_parser(self, subparsers):
        self.subparser = subparsers.add_parser(
            "logout", help="Log out from your Anaconda repository", description=__doc__
        )
        self.subparser.add_argument(
            "-f",
            "--force",
            dest="force",
            action="store_true",
            help="Remove relevant tokens event if not logged in. Default False",
        )

        self.subparser.set_defaults(main=self.main)



from unittest.mock import patch

import requests_mock

from repo_cli.commands.download import SubCommand


@patch("repo_cli.commands.base.SubCommandBase")
@patch("requests.post")
def test_download_notebook(mock_command_base, mock, tmp_path):
    mock_command_base.api.get_notebook_download_url.return_value = "http://testurl.com"
    subcommand = SubCommand(mock_command_base)
    my_file = tmp_path / "test.ipynb"

    with requests_mock.Mocker() as mock:
        mock.register_uri(
            "GET",
            "http://testurl.com",
            content="test".encode(),
            status_code=200,
        )
        subcommand.download_notebook("test_channel", "test_package", filename=my_file)

        assert my_file.read_text() == "test"



import logging
import socket
import sys
import threading
from collections import namedtuple
from functools import partial
from http.server import BaseHTTPRequestHandler, HTTPServer
from urllib.parse import parse_qs, urlencode, urlparse

import requests

logger = logging.getLogger("repo_cli")


Config = namedtuple(
    "Config",
    [
        "client_id",
        "port",
        "authorize_url",
        "token_url",
        "redirect_uri",
        "server",
        "verify_ssl",
    ],
)

HTML_SUCCESS_RESPONSE = """
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Anaconda Server - Token received</title>
    <style>
        body { text-align: center; padding: 150px; }
        h1 { font-size: 50px; }
        body { font: 20px Helvetica, sans-serif; color: #333; }
        article { display: block; text-align: left; width: 650px; margin: 0 auto; }
        a { color: #dc8100; text-decoration: none; }
        a:hover { color: #333; text-decoration: none; }
    </style>
</head>
<body>
    <article>
        <h1>Token received</h1>
        <div>
            <p>We successfully received a token.<br>You can close the browser window and continue to use the CLI</p>
            <p>&mdash; Anaconda</p>
        </div>
    </article>
</body>
</html>
"""

HTML_404_RESPONSE = """
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Anaconda Server - Not Found</title>
    <style>
        body { text-align: center; padding: 150px; }
        h1 { font-size: 50px; }
        body { font: 20px Helvetica, sans-serif; color: #333; }
        article { display: block; text-align: left; width: 650px; margin: 0 auto; }
        a { color: #dc8100; text-decoration: none; }
        a:hover { color: #333; text-decoration: none; }
    </style>
</head>
<body>
    <article>
        <h1>404</h1>
        <div>
            <p>Not Found</p>
        </div>
    </article>
</body>
</html>
"""


class RequestHandler(BaseHTTPRequestHandler):
    def __init__(self, config, *args, **kwargs):
        self.config = config
        super().__init__(*args, **kwargs)

    def do_GET(self):
        """The main handler for the auth server.

        To initiate the flow, we are openining the browser at http://localhost:PORT/get_token, which
        generates the redirect header to the OpenID Connect Provider/

        Once user is logged in on OpenID Connect provider, it's redirected back to the
        http://localhost:PORT/?code=CODE&.... location, query string contains the authorization
        code in `code` parameter.

        Next we need to exchange the code for a jwt token, calling the token endpoint.
        """
        if "/get_token" in self.path:
            qs = urlencode(
                {
                    "client_id": self.config.client_id,
                    "response_type": "code",
                    "redirect_uri": self.config.redirect_uri,
                }
            )
            redirect_url = "{authorize_url}?{qs}".format(
                authorize_url=self.config.authorize_url, qs=qs
            )
            return self.redirect(redirect_url)

        if "code=" in self.path:
            qs = parse_qs(urlparse(self.path).query)
            code = qs["code"][0]
            return self.process_code(code)

        self.response(404, HTML_404_RESPONSE)

    def process_code(self, code):
        """Exchanges the authorization code, send back by OpenID Connect provider for the token,
        calling the token endpoint with the proper client_id and redirect_uri.

        Args:
            code (str): authorization_code string

        Returns:
            None, stops the webserver by exiting and stopping the thread.
        """
        token_url = self.config.token_url
        data = {
            "grant_type": "authorization_code",
            "code": code,
            "client_id": self.config.client_id,
            "redirect_uri": self.config.redirect_uri,
        }
        r = requests.post(token_url, data, verify=self.config.verify_ssl)

        self.response(200, HTML_SUCCESS_RESPONSE)
        if r.status_code == 200:
            access_token = r.json()["access_token"]
            self.config.server.access_token = access_token
            logger.info("Received access_token")
        else:
            logger.error(
                "Accessed %s with %s and got error %s", token_url, data, r.text
            )
        sys.exit()

    def redirect(self, location):
        self.send_response(302)
        self.send_header("Location", location)
        self.end_headers()

    def response(self, status, data):
        self.send_response(status)
        self.send_header("Content-Type", "text/html")
        self.end_headers()
        self.wfile.write(data.encode())

    def log_message(self, format, *args):
        pass


class WebServer:
    def __init__(self, client_id, openid_configuration_url, port=None, verify_ssl=None):
        self.client_id = client_id
        self.verify_ssl = verify_ssl

        openid_conf = requests.get(
            openid_configuration_url, verify=self.verify_ssl
        ).json()
        self.authorize_url = openid_conf["authorization_endpoint"]
        self.token_url = openid_conf["token_endpoint"]
        if port is not None:
            self.port = port
        else:
            self.port = self.find_unused_port()
        self._access_token = None

    @property
    def access_token(self):
        return self._access_token

    @access_token.setter
    def access_token(self, value):
        if value:
            self._access_token = value

    def start(self):
        server_address = ("", self.port)
        config = Config(
            port=self.port,
            client_id=self.client_id,
            authorize_url=self.authorize_url,
            token_url=self.token_url,
            redirect_uri=self.localhost_url("/"),
            server=self,
            verify_ssl=self.verify_ssl,
        )
        handler = partial(RequestHandler, config)
        server = HTTPServer(server_address, handler)
        thread = threading.Thread(target=server.serve_forever)
        thread.daemon = True
        return thread

    def find_unused_port(self):
        """Returns an unused port number on localhost. Will be used for webserver port."""
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            s.bind(("127.0.0.1", 0))
            return s.getsockname()[1]

    def localhost_url(self, path="/get_token"):
        return "http://localhost:{port}{path}".format(port=self.port, path=path)



"""
Manage your Anaconda repository channels.
"""

from __future__ import print_function, unicode_literals

import argparse
from pprint import pformat

from .. import errors
from ..utils import format
from ..utils.artifacts import SimplePackageSpec
from .base import SubCommandBase


class SubCommand(SubCommandBase):
    name = "channel"

    # flake8: noqa: C901
    def main(self):
        self.log.info("")
        if self.args.create:
            channel = self.args.create
            if not channel:
                msg = (
                    "Channel name not specified. Please use -n or -o to specify your channel.\n"
                    "Use --help for help."
                )
                self.log.info(msg)
                raise errors.RepoCLIError(msg)

            self.api.create_channel(channel)
            self.log.info("Channel %s successfully created" % channel)
        elif self.args.remove:
            self.api.remove_channel(self.args.remove)
        elif self.args.list:
            self.list_user_channels()
        elif self.args.list_packages:
            for channel_name in self.args.list_packages:
                self.show_channel_packages(
                    channel_name,
                    self.args.offset if self.args.offset else None,
                    self.args.limit,
                    self.args.full_details,
                )
        elif self.args.list_files:
            for spec in self.args.list_files:
                self.show_channel_files(
                    spec, family=self.args.family, full_details=self.args.full_details
                )
        elif self.args.show:
            self.show_channel(self.args.show, full_details=self.args.full_details)
        elif self.args.history:
            self.show_channel_history(
                self.args.history,
                self.args.offset,
                self.args.limit,
                self.args.full_details,
            )
        elif self.args.lock:
            channel = self.args.lock
            msg = "{} {} is now locked".format(self.name.title(), channel)
            self.api.update_channel(channel, privacy="private", success_message=msg)
        elif self.args.soft_lock:
            channel = self.args.soft_lock
            msg = "{} {} is now soft-locked".format(self.name.title(), channel)
            self.api.update_channel(
                channel, privacy="authenticated", success_message=msg
            )
        elif self.args.unlock:
            channel = self.args.unlock
            msg = "{} {} is now unlocked".format(self.name.title(), channel)
            self.api.update_channel(channel, privacy="public", success_message=msg)
        elif self.args.freeze:
            channel = self.args.freeze
            msg = "{} {} is now frozen".format(self.name.title(), channel)
            self.api.update_channel(
                channel, indexing_behavior="frozen", success_message=msg
            )
        elif self.args.unfreeze:
            channel = self.args.unfreeze
            msg = "{} {} is now unfrozen".format(self.name.title(), channel)
            self.api.update_channel(
                channel, indexing_behavior="default", success_message=msg
            )
        else:
            raise NotImplementedError()

    def filter_response(self, data, offset=None, limit=None):
        filtered_data = data[offset : (limit + offset if limit is not None else None)]
        return filtered_data

    def show_channel_packages(self, channel_name, offset, limit, full_details):
        data = self.api.get_channel_artifacts(channel_name, offset, limit)
        packages = data.get("items", [])

        self.log.info("")
        self.log.info(
            "Total packages matching spec %s found: %s\n"
            % (channel_name, len(packages))
        )

        if full_details:
            for package in packages:
                self.show_package_detail(package)
            self.log.info("")
        else:
            formatter = format.PackagesFormatter(log=self.log)
            formatter.format(
                packages=packages,
                metadata={"offset": offset, "total_count": data["total_count"]},
                simple=True,
            )

    def show_package_detail(self, package):
        keymap = {
            "download_count": "# of downloads",
            "file_count": "# of files",
        }
        pack = dict(package)
        pack.update(package["metadata"])
        resp = ["---------------"]

        for key in ["name", "file_count", "download_count", "license", "description"]:
            label = keymap.get(key, key)
            value = pack.get(key, "")
            resp.append("%s: %s" % (label, value))

        self.log.info("\n".join(resp))

    def show_channel_files(self, spec, family, full_details=False):
        packages, total_count = self.api.get_channel_artifacts_files(
            spec.channel,
            family=family,
            package=spec.package,
            version=spec.version,
            filename=spec.filename,
        )
        if not packages:
            self.log.warning(
                "No files matches were found for the provided spec: %s\n" % (spec)
            )
            return

        formatter = format.PackageFilesFormatter
        if "cve_score" in packages[0]:
            formatter = format.PackageFilesFormatterWithCVE

        packages = self.filter_response(packages, self.args.offset, self.args.limit)
        total_count = len(packages)
        self.log.info(
            "Found %s files matching the specified spec %s:" % (total_count, spec)
        )

        if full_details:
            self._show_channel_files_full_format(packages)
        else:
            self.log.info("")
            self.log.info(formatter.format_list(packages))
            self.log.info("")

    def _show_channel_files_full_format(self, packages):
        for package in packages:
            self.log.info("----------------")
            self.log.info(pformat(package))

    def show_channel(self, channel, full_details=False):
        channel_data = self.api.get_channel(channel)
        if full_details and not self.api.is_subchannel(channel):
            channel_data["subchannels"] = self.api.get_channel_subchannels(channel)

        self.show_channel_detail(channel_data)

    def show_channel_history(self, channel, offset, limit, full_details):
        data = self.api.get_channel_history(channel, offset, limit)
        self.log.info(
            format.HistoryFormatter.format_list(data["items"], not full_details)
        )
        self.log.info("")

    def show_channel_detail(self, data):
        resp = ["Channel details:", ""]
        keymap = {
            "download_count": "# of downloads",
            "artifact_count": "# of artifacts",
            "mirror_count": "# mirrors",
            "subchannel_count": "# of subchannels",
        }

        for key in [
            "name",
            "description",
            "privacy",
            "artifact_count",
            "download_count",
            "mirror_count",
            "subchannel_count",
            "created",
            "updated",
            "indexing_behavior",
        ]:
            label = keymap.get(key, key)
            value = data.get(key, "")
            resp.append("\t%s: %s" % (label, value))

        try:
            owners = ", ".join(data.get("owners", [""]))
        except TypeError:
            owners = ""
        resp.append("\towners: %s" % owners)
        if "subchannels" in data:
            resp.append("")
            resp.append("\tSubchannels:")
            resp.append("\t------------")

            resp.append(format.PackagesFormatter.format_channel_header())
            for subchannel in data["subchannels"]["items"]:
                resp.append(format.PackagesFormatter.format_channel(subchannel))

        resp.append("")
        self.log.info("\n".join(resp))

    def list_user_channels(self):
        self.show_list_channels(self.api.list_user_channels())

    def show_list_channels(self, data):
        resp = ["Channels available to the user:", ""]
        keymap = {"download_count": "downloads", "artifact_count": "artifacts"}
        cols_ = ["name", "privacy", "description", "artifact_count", "download_count"]
        cols = [keymap.get(key, key) for key in cols_]
        resp.append("\t".join(cols))

        for ch in data["items"]:
            resp.append("\t".join([str(ch.get(key, "")) for key in cols_]))

        resp.append("")
        self.log.info("\n".join(resp))

    def add_parser(self, subparsers):
        subparser = subparsers.add_parser(
            self.name,
            help="Manage your Anaconda repository {}s".format(self.name),
            formatter_class=argparse.RawDescriptionHelpFormatter,
            description=__doc__,
        )
        subparser.add_argument(
            "--family",
            default=None,
            help="Artifact family (i.e.: conda, python, cran, anaconda_project, anaconda_env, notebook). "
            "ONLY USED IN COMBINATION WITH --list-files, ignored otherwise.",
        )
        subparser.add_argument(
            "--full-details",
            help="Prints full file details. ONLY USED IN COMBINATION WITH --list-files, --list-packages "
            "or --show ignored otherwise.",
            action="store_true",
            default=False,
        )
        subparser.add_argument(
            "-o",
            "--offset",
            default=0,
            type=int,
            help="Offset when displaying the results",
        )
        subparser.add_argument(
            "-l",
            "--limit",
            default=50,
            type=int,
            help="Limit when displaying the results",
        )

        group = subparser.add_mutually_exclusive_group(required=True)

        group.add_argument(
            "--create",
            # action='store_true',
            metavar=self.name.upper(),
            help="Create a new {}".format(self.name),
        )
        group.add_argument(
            "--list",
            action="store_true",
            help="List all {}s for a user".format(self.name),
        )
        group.add_argument(
            "--list-packages",
            help="Channel written as <channel>[/<subchannel>]",
            metavar="CHANNEL",
            nargs="+",
        )
        group.add_argument(
            "--list-files",
            help="Package written as <channel>[/<subchannel>]::<package>[/<version>[/<filename>]]",
            metavar="PACKAGE",
            type=SimplePackageSpec.from_string,
            nargs="+",
        )
        group.add_argument(
            "--show",
            metavar=self.name.upper(),
            help="Show information about the {}".format(self.name),
        )
        group.add_argument(
            "--history",
            metavar=self.name.upper(),
            help="Show historical events about the {}".format(self.name),
        )
        group.add_argument(
            "--lock", metavar=self.name.upper(), help="Lock a {}".format(self.name)
        )
        group.add_argument(
            "--soft-lock",
            metavar=self.name.upper(),
            help="Soft Lock a {}, so that only authenticated users can see it.".format(
                self.name
            ),
        )
        group.add_argument(
            "--unlock", metavar=self.name.upper(), help="Unlock a {}".format(self.name)
        )
        group.add_argument(
            "--remove", metavar=self.name.upper(), help="Remove a {}".format(self.name)
        )
        group.add_argument(
            "--freeze", metavar=self.name.upper(), help="Freeze a {}".format(self.name)
        )
        group.add_argument(
            "--unfreeze",
            metavar=self.name.upper(),
            help="Unfreeze a {}".format(self.name),
        )

        subparser.set_defaults(main=self.main)



from __future__ import absolute_import, print_function, unicode_literals

import collections
import itertools
import logging
import os
import stat
import warnings
from os.path import abspath, dirname, exists, expanduser, isdir, isfile, join
from string import Template

try:
    from collections.abc import Mapping
except ImportError:
    from collections import Mapping


try:
    from urllib import quote_plus
except ImportError:
    from urllib.parse import quote_plus

# from binstar_client.utils.appdirs import AppDirs, EnvAppDirs
# from binstar_client.errors import BinstarError
from ..errors import RepoCLIError
from .appdirs import AppDirs, EnvAppDirs
from .conda import CONDA_PREFIX, CONDA_ROOT
from .yaml import yaml_dump, yaml_load

logger = logging.getLogger("repo_cli")


def expandvars(path):
    environ = dict(CONDA_ROOT=CONDA_ROOT, CONDA_PREFIX=CONDA_PREFIX)
    environ.update(os.environ)
    return Template(path).safe_substitute(**environ)


def expand(path):
    return abspath(expanduser(expandvars(path)))


CONFIG_FILENAME = "repo-cli-config.yaml"
if "REPO_CONFIG_DIR" in os.environ:
    dirs = EnvAppDirs(".conda", os.environ["REPO_CONFIG_DIR"])
    USER_CONFIG = join(dirs.user_data_dir, CONFIG_FILENAME)
else:
    # we need to keep it to binstar to be compatible with conda. ContinuuIO is important for
    # Windows users
    dirs = AppDirs("binstar", "ContinuumIO")
    USER_CONFIG = expand("~/.conda/" + CONFIG_FILENAME)
# if 'BINSTAR_CONFIG_DIR' in os.environ:
#     dirs = EnvAppDirs('binstar', 'ContinuumIO', os.environ['BINSTAR_CONFIG_DIR'])
#     USER_CONFIG = join(dirs.user_data_dir, 'config.yaml')
# else:
#     dirs = AppDirs('binstar', 'ContinuumIO')
#     USER_CONFIG = expand('~/.continuum/anaconda-client/config.yaml')


# Package types used in upload/download
PACKAGE_TYPES = {
    "env": "Environment",
    "ipynb": "Notebook",
    "conda": "Conda Package",
    "pypi": "Python Package",
    "project": "Anaconda Project",
}

UPLOAD_TYPE_MAPPING = {
    "conda": "conda1",
    "env": "anaconda_env",
    "ipynb": "jupyter_notebook",
    "project": "anaconda_project",
    "pypi": "bdist_wheel",
    "sbom": "sbom",
}

USER_LOGDIR = dirs.user_log_dir
SITE_CONFIG = expand("$CONDA_ROOT/etc/anaconda-client/%s" % (CONFIG_FILENAME,))
SYSTEM_CONFIG = SITE_CONFIG

OIDC_CLIENT_ID = "repo-service-cli"

DEFAULT_CONFIG = {
    "sites": {},
    "auto_register": True,
    "default_site": None,
    "url": None,
    "ssl_verify": True,
    "oauth2": False,
}

CONFIGURATION_KEYS = [
    "auto_register",
    "default_site",
    "sites",
    "url",
    "ssl_verify",
    "oauth2",
]

SEARCH_PATH = (
    "~/.conda/",
    dirs.site_data_dir,
    "/etc/anaconda-client/",
    "$CONDA_ROOT/etc/anaconda-client/",
    dirs.user_data_dir,
    "~/.continuum/anaconda-client/",
    "$CONDA_PREFIX/etc/anaconda-client/",
)


def recursive_update(config, update_dict):
    for update_key, updated_value in update_dict.items():
        if isinstance(updated_value, Mapping):
            updated_value_dict = recursive_update(
                config.get(update_key, {}), updated_value
            )
            config[update_key] = updated_value_dict
        else:
            config[update_key] = update_dict[update_key]

    return config


TOKEN_DIRS = [
    dirs.user_data_dir,
    join(dirname(USER_CONFIG), "tokens"),
]
TOKEN_DIR = TOKEN_DIRS[-1]


def store_token(token, args):
    config = get_config(site=args and args.site)

    for token_dir in TOKEN_DIRS:
        url = config.get("url")

        if not isdir(token_dir):
            os.makedirs(token_dir)
        tokenfile = join(token_dir, "%s.token" % quote_plus(url))

        if isfile(tokenfile):
            os.unlink(tokenfile)

        logger.debug(f"Saving token {token} to {tokenfile}")
        with open(tokenfile, "w") as fd:
            logger.debug(f"Saving token {token} to {tokenfile}")
            fd.write(token)
        os.chmod(tokenfile, stat.S_IWRITE | stat.S_IREAD)

        # TODO: This is a hack
        tokenfile = join(token_dir, "%s.token" % quote_plus(join(url, "repo")))
        logger.debug(f"Saving token {token} to {tokenfile}")
        with open(tokenfile, "w") as fd:
            logger.debug(f"Saving token {token} to {tokenfile}")
            fd.write(token)
        os.chmod(tokenfile, stat.S_IWRITE | stat.S_IREAD)


def load_token(site):
    config = get_config(site=site)
    url = config.get("url")
    if not url:
        logger.debug(
            "Repository URL is not set. Please configure. Skipping token check"
        )
        return

    for token_dir in TOKEN_DIRS:
        logger.debug(
            "[LOAD TOKEN] Looking for token in {token_dir}".format(token_dir=token_dir)
        )
        tokenfile = join(token_dir, "%s.token" % quote_plus(url))
        _is_file = exists(tokenfile)

        if _is_file:
            logger.debug("[LOAD TOKEN] Found login token: {}".format(tokenfile))
            with open(tokenfile) as fd:
                token = fd.read().strip()

            if token:
                return token
            else:
                logger.debug("Token file is empty: {}".format(tokenfile))
                logger.debug("Removing file: {}".format(tokenfile))
                os.unlink(tokenfile)
        else:
            logger.debug(
                "[LOAD TOKEN] Token file {tokenfile} not found.".format(
                    tokenfile=tokenfile
                )
            )


def remove_token(args):
    config = get_config(site=args and args.site)
    url = config.get("url")

    for token_dir in TOKEN_DIRS:
        tokenfile = join(token_dir, "%s.token" % quote_plus(url))
        if isfile(tokenfile):
            os.unlink(tokenfile)

        tokenfile = join(token_dir, "%s.token" % quote_plus(join(url, "repo")))
        if isfile(tokenfile):
            os.unlink(tokenfile)


def load_config(config_file):
    if exists(config_file):
        with open(config_file) as fd:
            data = yaml_load(fd)
            if data:
                return data

    return {}


def load_file_configs(search_path):
    def _file_yaml_loader(fullpath):
        assert (
            fullpath.endswith(".yml")
            or fullpath.endswith(".yaml")
            or fullpath.endswith("anacondarc")
        ), fullpath
        yield fullpath, load_config(fullpath)

    def _dir_yaml_loader(fullpath):
        for filename in os.listdir(fullpath):
            if filename == CONFIG_FILENAME:
                filepath = join(fullpath, filename)
                yield filepath, load_config(filepath)

    # map a stat result to a file loader or a directory loader
    _loader = {
        stat.S_IFREG: _file_yaml_loader,
        stat.S_IFDIR: _dir_yaml_loader,
    }

    def _get_st_mode(path):
        # stat the path for file type, or None if path doesn't exist
        try:
            return stat.S_IFMT(os.stat(path).st_mode)
        except OSError:
            return None

    expanded_paths = [expand(path) for path in search_path]
    stat_paths = (_get_st_mode(path) for path in expanded_paths)
    load_paths = (
        _loader[st_mode](path)
        for path, st_mode in zip(expanded_paths, stat_paths)
        if st_mode is not None
    )
    raw_data = collections.OrderedDict(
        kv for kv in itertools.chain.from_iterable(load_paths)
    )

    return raw_data


def get_config(site=None):
    config = DEFAULT_CONFIG.copy()

    file_configs = load_file_configs(SEARCH_PATH)
    for fn in file_configs:
        recursive_update(config, file_configs[fn])

    site = site or config.get("default_site")
    sites = config.get("sites", {})

    if site:
        site = str(site)

        if site not in sites:
            logger.warning('Site alias "%s" does not exist in the config file', site)
        else:
            # This takes whatever keys are set for the site into the top level of the config dict
            recursive_update(config, sites.get(site, {}))

    return config


def save_config(data, config_file):
    data_dir = dirname(config_file)

    try:
        if not exists(data_dir):
            os.makedirs(data_dir)

        with open(config_file, "w") as fd:
            yaml_dump(data, stream=fd)
    except EnvironmentError as exc:
        raise RepoCLIError(
            "%s: %s"
            % (
                exc.filename,
                exc.strerror,
            )
        )


def set_config(data, user=True):
    warnings.warn("Use save_config instead of set_config", DeprecationWarning)
    save_config(data, USER_CONFIG if user else SYSTEM_CONFIG)
